@@ -275,13 +275,29 @@ static MVMObject * decont_arg(MVMThreadContext *tc, MVMObject *arg) {
     dest = box; \
 } while (0)
 
+#define autobox_int(tc, target, result, dest) do { \
+    MVMObject *box, *box_type; \
+    MVMint64 result_int = result; \
+    box_type = target->static_info->body.cu->body.hll_config->int_box_type; \
+    dest = MVM_intcache_get(tc, box_type, result_int); \
+    if (!dest) { \
+        box = REPR(box_type)->allocate(tc, STABLE(box_type)); \
+        MVM_gc_root_temp_push(tc, (MVMCollectable **)&box); \
+        if (REPR(box)->initialize) \
+            REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box)); \
+        REPR(box)->box_funcs.set_int(tc, STABLE(box), box, OBJECT_BODY(box), result_int); \
+        MVM_gc_root_temp_pop(tc); \
+        dest = box; \
+    } \
+} while (0)
+
 #define autobox_switch(tc, result) do { \
     if (result.exists) { \
         switch (result.flags & MVM_CALLSITE_ARG_MASK) { \
             case MVM_CALLSITE_ARG_OBJ: \
                 break; \
             case MVM_CALLSITE_ARG_INT: \
-                autobox(tc, tc->cur_frame, result.arg.i64, int_box_type, 0, set_int, result.arg.o); \
+                autobox_int(tc, tc->cur_frame, result.arg.i64, result.arg.o); \
                 break; \
             case MVM_CALLSITE_ARG_NUM: \
                 autobox(tc, tc->cur_frame, result.arg.n64, num_box_type, 0, set_num, result.arg.o); \
@@ -510,6 +526,24 @@ void MVM_args_assert_void_return_ok(MVMThreadContext *tc, MVMint32 frameless) {
         OBJECT_BODY(result), reg, MVM_reg_obj); \
 } while (0)
 
+#define box_slurpy_pos_int(tc, type, result, box, value, reg, box_type_obj, name, set_func) do { \
+    type = (*(tc->interp_cu))->body.hll_config->box_type_obj; \
+    if (!type || IS_CONCRETE(type)) { \
+        MVM_exception_throw_adhoc(tc, "Missing hll " name " box type"); \
+    } \
+    box = MVM_intcache_get(tc, type, value); \
+    if (!box) { \
+        box = REPR(type)->allocate(tc, STABLE(type)); \
+        if (REPR(box)->initialize) \
+            REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box)); \
+        REPR(box)->box_funcs.set_func(tc, STABLE(box), box, \
+            OBJECT_BODY(box), value); \
+    } \
+    reg.o = box; \
+    REPR(result)->pos_funcs.push(tc, STABLE(result), result, \
+        OBJECT_BODY(result), reg, MVM_reg_obj); \
+} while (0)
+
 MVMObject * MVM_args_slurpy_positional(MVMThreadContext *tc, MVMArgProcContext *ctx, MVMuint16 pos) {
     MVMObject *type = (*(tc->interp_cu))->body.hll_config->slurpy_array_type, *result = NULL, *box = NULL;
     MVMArgInfo arg_info;
@@ -541,7 +575,7 @@ MVMObject * MVM_args_slurpy_positional(MVMThreadContext *tc, MVMArgProcContext *
                 break;
             }
             case MVM_CALLSITE_ARG_INT:{
-                box_slurpy_pos(tc, type, result, box, arg_info.arg.i64, reg, int_box_type, "int", set_int);
+                box_slurpy_pos_int(tc, type, result, box, arg_info.arg.i64, reg, int_box_type, "int", set_int);
                 break;
             }
             case MVM_CALLSITE_ARG_NUM: {
