@@ -108,12 +108,15 @@ static MVMint32 search_frame_handlers(MVMThreadContext *tc, MVMFrame *f,
             }
         }
     } else {
+        MVMint32 num_handlers = f->spesh_cand
+            ? f->spesh_cand->num_handlers
+            : f->static_info->body.num_handlers;
         MVMint32 pc;
         if (f == tc->cur_frame)
             pc = (MVMuint32)(*tc->interp_cur_op - *tc->interp_bytecode_start);
         else
             pc = (MVMuint32)(f->return_address - f->effective_bytecode);
-        for (i = 0; i < f->static_info->body.num_handlers; i++) {
+        for (i = 0; i < num_handlers; i++) {
             MVMFrameHandler  *fh = &f->effective_handlers[i];
             if (!handler_can_handle(f, fh, cat, payload))
                 continue;
@@ -506,8 +509,13 @@ void MVM_exception_throwobj(MVMThreadContext *tc, MVMuint8 mode, MVMObject *ex_o
 
     if (!ex->body.category)
         ex->body.category = MVM_EX_CAT_CATCH;
-    if (resume_result)
+    if (resume_result) {
         ex->body.resume_addr = *tc->interp_cur_op;
+        /* Ensure that the jit resume label is stored. The throwish
+         * control guard should ensure that the jit entry label point to
+         * a position just after throwing. */
+        ex->body.jit_resume_label = tc->cur_frame->jit_entry_label;
+    }
     lh = search_for_handler_from(tc, tc->cur_frame, mode, ex->body.category, ex->body.payload);
     if (lh.frame == NULL)
         panic_unhandled_ex(tc, ex);
@@ -558,7 +566,8 @@ void MVM_exception_resume(MVMThreadContext *tc, MVMObject *ex_obj) {
     MVM_frame_dec_ref(tc, ah->frame);
     MVM_free(ah);
 
-    /* Unwind to the thrower of the exception; set PC. */
+    /* Unwind to the thrower of the exception; set PC and jit entry label. */
+    target->jit_entry_label = ex->body.jit_resume_label;
     MVM_frame_unwind_to(tc, target, ex->body.resume_addr, 0, NULL);
 }
 
