@@ -331,19 +331,22 @@ MVMObject * MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result_type, MV
 MVMObject * MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result_type, MVMObject *a, MVMObject *b) { \
     MVMP6bigintBody *ba, *bb, *bc; \
     MVMObject *result; \
-    MVMROOT(tc, a, { \
-    MVMROOT(tc, b, { \
-        result = MVM_repr_alloc_init(tc, result_type);\
-    }); \
-    }); \
     ba = get_bigint_body(tc, a); \
     bb = get_bigint_body(tc, b); \
-    bc = get_bigint_body(tc, result); \
     if (MVM_BIGINT_IS_BIG(ba) || MVM_BIGINT_IS_BIG(bb)) { \
         mp_int *tmp[2] = { NULL, NULL }; \
-        mp_int *ia = force_bigint(ba, tmp); \
-        mp_int *ib = force_bigint(bb, tmp); \
-        mp_int *ic = MVM_malloc(sizeof(mp_int)); \
+        mp_int *ia, *ib, *ic; \
+        MVMROOT(tc, a, { \
+        MVMROOT(tc, b, { \
+            result = MVM_repr_alloc_init(tc, result_type);\
+        }); \
+        }); \
+        ba = get_bigint_body(tc, a); \
+        bb = get_bigint_body(tc, b); \
+        bc = get_bigint_body(tc, result); \
+        ia = force_bigint(ba, tmp); \
+        ib = force_bigint(bb, tmp); \
+        ic = MVM_malloc(sizeof(mp_int)); \
         mp_init(ic); \
         mp_##opname(ia, ib, ic); \
         store_bigint_result(bc, ic); \
@@ -354,6 +357,11 @@ MVMObject * MVM_bigint_##opname(MVMThreadContext *tc, MVMObject *result_type, MV
         MVMint64 sa = ba->u.smallint.value; \
         MVMint64 sb = bb->u.smallint.value; \
         SMALLINT_OP; \
+        result = MVM_intcache_get(tc, result_type, sc); \
+        if (result) \
+            return result; \
+        result = MVM_repr_alloc_init(tc, result_type);\
+        bc = get_bigint_body(tc, result); \
         store_int64_result(bc, sc); \
     } \
     return result; \
