@@ -18,6 +18,17 @@ static void append(DumpStr *ds, char *to_add) {
     ds->pos += len;
 }
 
+static size_t tell_ds(DumpStr *ds) {
+    return ds->pos;
+}
+
+static void rewind_ds(DumpStr *ds, size_t target) {
+    if (ds->pos > target) {
+        ds->pos = target;
+        ds->buffer[ds->pos + 1] = '\0';
+    }
+}
+
 /* Formats a string and then appends it. */
 MVM_FORMAT(printf, 2, 3)
 static void appendf(DumpStr *ds, const char *fmt, ...) {
@@ -133,13 +144,16 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
             ann = ann->next;
         }
 
-        appendf(ds, "      %s ", cur_ins->info->name);
+        appendf(ds, "      %-15s ", cur_ins->info->name);
         if (cur_ins->info->opcode == MVM_SSA_PHI) {
             for (i = 0; i < cur_ins->info->num_operands; i++) {
+                MVMint16 orig = cur_ins->operands[i].reg.orig;
+                MVMint16 regi = cur_ins->operands[i].reg.i;
                 if (i)
                     append(ds, ", ");
-                appendf(ds, "r%d(%d)", cur_ins->operands[i].reg.orig,
-                    cur_ins->operands[i].reg.i);
+                if (orig < 10) append(ds, " ");
+                if (regi < 10) append(ds, " ");
+                appendf(ds, "r%d(%d)", orig, regi);
             }
         }
         else {
@@ -148,10 +162,14 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                     append(ds, ", ");
                 switch (cur_ins->info->operands[i] & MVM_operand_rw_mask) {
                     case MVM_operand_read_reg:
-                    case MVM_operand_write_reg:
-                        appendf(ds, "r%d(%d)", cur_ins->operands[i].reg.orig,
-                            cur_ins->operands[i].reg.i);
+                    case MVM_operand_write_reg: {
+                        MVMint16 orig = cur_ins->operands[i].reg.orig;
+                        MVMint16 regi = cur_ins->operands[i].reg.i;
+                        if (orig < 10) append(ds, " ");
+                        if (regi < 10) append(ds, " ");
+                        appendf(ds, "r%d(%d)", orig, regi);
                         break;
+                    }
                     case MVM_operand_read_lex:
                     case MVM_operand_write_lex: {
                         MVMStaticFrameBody *cursor = &g->sf->body;
@@ -177,9 +195,13 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                     case MVM_operand_literal: {
                         MVMuint32 type = cur_ins->info->operands[i] & MVM_operand_type_mask;
                         switch (type) {
-                        case MVM_operand_ins:
-                            appendf(ds, "BB(%d)", cur_ins->operands[i].ins_bb->idx);
+                        case MVM_operand_ins: {
+                            MVMint32 bb_idx = cur_ins->operands[i].ins_bb->idx;
+                            if (bb_idx < 100) append(ds, " ");
+                            if (bb_idx < 10)  append(ds, " ");
+                            appendf(ds, "BB(%d)", bb_idx);
                             break;
+                        }
                         case MVM_operand_int8:
                             appendf(ds, "liti8(%"PRId8")", cur_ins->operands[i].lit_i8);
                             break;
@@ -270,7 +292,9 @@ static void dump_facts(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
         for (j = 0; j < num_facts; j++) {
             MVMint32 usages = g->facts[i][j].usages;
             MVMint32 flags  = g->facts[i][j].flags;
-            appendf(ds, "    r%d(%d): usages=%d, flags=%d", i, j, usages, flags);
+            if (i < 10) append(ds, " ");
+            if (j < 10) append(ds, " ");
+            appendf(ds, "    r%d(%d): usages=%d, flags=%-5d", i, j, usages, flags);
             if (flags & 1) {
                 append(ds, " KnTyp");
             }
@@ -312,6 +336,73 @@ static void dump_facts(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
     }
 }
 
+/* Dumps a table of all logged values */
+static void dump_log_values(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
+    MVMint16 log_index;
+    MVMint16 seen_table_size =  g->num_log_slots * MVM_SPESH_LOG_RUNS;
+    size_t   ds_pos_before = tell_ds(ds);
+    MVMint16 interesting = 0;
+
+    MVMCollectable **seen_table = alloca(sizeof(MVMCollectable *) *seen_table_size);
+    memset(seen_table, 0, sizeof(MVMCollectable *) * seen_table_size);
+
+    append(ds, "Logged values:\n");
+
+    for (log_index = 0; log_index < g->num_log_slots; log_index++) {
+        MVMint16 run_index;
+
+        appendf(ds, "    % 3d ", log_index);
+
+        for (run_index = 0; run_index < MVM_SPESH_LOG_RUNS; run_index++) {
+            MVMuint16       log_slot = log_index * MVM_SPESH_LOG_RUNS + run_index;
+            MVMCollectable *log_obj  = g->log_slots[log_slot];
+            MVMint16        log_obj_idx;
+
+            if (log_obj) {
+                for (log_obj_idx = 0; log_obj_idx < seen_table_size; log_obj_idx++) {
+                    if (seen_table[log_obj_idx] == log_obj) {
+                        break;
+                    } else if (seen_table[log_obj_idx] == 0) {
+                        seen_table[log_obj_idx] = log_obj;
+                        break;
+                    }
+                }
+
+                appendf(ds, "% 4d  ", log_obj_idx + 1);
+                interesting = 1;
+            } else {
+                appendf(ds, "% 4s  ", "_");
+            }
+
+        }
+
+        append(ds, "\n");
+    }
+    append(ds, "\n");
+
+    for (log_index = 0; log_index < seen_table_size && seen_table[log_index]; log_index++) {
+        appendf(ds, "    %d: %p", log_index + 1, seen_table[log_index]);
+        if (STABLE(seen_table[log_index])->REPR->ID == MVM_REPR_ID_P6int) {
+            if (IS_CONCRETE(seen_table[log_index]))
+                appendf(ds, " P6int(%d)", MVM_repr_get_int(tc, (MVMObject*)seen_table[log_index]));
+            else
+                append(ds, " P6int(type object)");
+        } else {
+            append(ds, " ");
+            append(ds, (char *)STABLE(seen_table[log_index])->REPR->name);
+            if (!IS_CONCRETE(seen_table[log_index]))
+                append(ds, "(type object)");
+        }
+        append(ds, "\n");
+    }
+
+    append(ds, "\n");
+
+    if (!interesting) {
+        rewind_ds(ds, ds_pos_before);
+    }
+}
+
 static void dump_callsite(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
     MVMuint16 i;
     appendf(ds, "Callsite %p (%d args, %d pos)\n", g->cs, g->cs->arg_count, g->cs->num_pos);
@@ -325,6 +416,30 @@ static void dump_callsite(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
     append(ds, "\n");
 }
 
+static void dump_arg_guards(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
+    MVMuint16 i;
+    appendf(ds, "%d argument guards\n", g->num_arg_guards);
+
+    for (i = 0; i < g->num_arg_guards; i++) {
+        MVMSpeshGuard *guard = &g->arg_guards[i];
+        switch (guard->kind) {
+        case MVM_SPESH_GUARD_CONC:
+            appendf(ds, "  concrete(%d)\n", guard->slot);
+            break;
+        case MVM_SPESH_GUARD_TYPE:
+            appendf(ds, "  type(%d, %p)\n", guard->slot, guard->match);
+            break;
+        case MVM_SPESH_GUARD_DC_CONC:
+            appendf(ds, "  deconted_concrete(%d)\n", guard->slot);
+            break;
+        case MVM_SPESH_GUARD_DC_TYPE:
+            appendf(ds, "  deconted_type(%d, %p)\n", guard->slot, guard->match);
+            break;
+        }
+    }
+    append(ds, "\n");
+}
+
 static void dump_fileinfo(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
     MVMBytecodeAnnotation *ann = MVM_bytecode_resolve_annotation(tc, &g->sf->body, 0);
     MVMCompUnit            *cu = g->sf->body.cu;
@@ -363,7 +478,9 @@ char * MVM_spesh_dump(MVMThreadContext *tc, MVMSpeshGraph *g) {
     append(&ds, ")\n");
     if (g->cs)
         dump_callsite(tc, &ds, g);
-    else
+    if (g->num_arg_guards)
+        dump_arg_guards(tc, &ds, g);
+    if (!g->cs && !g->num_arg_guards)
         append(&ds, "\n");
 
     /* Go over all the basic blocks and dump them. */
@@ -377,6 +494,16 @@ char * MVM_spesh_dump(MVMThreadContext *tc, MVMSpeshGraph *g) {
     append(&ds, "\nFacts:\n");
     dump_facts(tc, &ds, g);
 
+    if (g->num_spesh_slots || g->num_log_slots) {
+        append(&ds, "\nStats:\n");
+        appendf(&ds, "    %d spesh slots\n", g->num_spesh_slots);
+        appendf(&ds, "    %d log slots\n", g->num_log_slots);
+    }
+
+    if (g->num_log_slots) {
+        dump_log_values(tc, &ds, g);
+    }
+
     append(&ds, "\n");
     append_null(&ds);
     return ds.buffer;
