@@ -45,6 +45,17 @@ static void copy_facts(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshOperand t
     tfacts->log_guard     = ffacts->log_guard;
 }
 
+/* Adds a value into a spesh slot and returns its index.
+ * If a spesh slot already holds this value, return that instead */
+MVMint16 MVM_spesh_add_spesh_slot_try_reuse(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCollectable *c) {
+    MVMint16 prev_slot;
+    for (prev_slot = 0; prev_slot < g->num_spesh_slots; prev_slot++) {
+        if (g->spesh_slots[prev_slot] == c)
+            return prev_slot;
+    }
+    return MVM_spesh_add_spesh_slot(tc, g, c);
+}
+
 /* Adds a value into a spesh slot and returns its index. */
 MVMint16 MVM_spesh_add_spesh_slot(MVMThreadContext *tc, MVMSpeshGraph *g, MVMCollectable *c) {
     if (g->num_spesh_slots >= g->alloc_spesh_slots) {
@@ -186,7 +197,7 @@ static void optimize_gethow(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns
     if (how_obj) {
         MVMSpeshFacts *how_facts;
         /* Transform gethow lookup to spesh slot lookup */
-        MVMint16 spesh_slot = MVM_spesh_add_spesh_slot(tc, g, (MVMCollectable*)how_obj);
+        MVMint16 spesh_slot = MVM_spesh_add_spesh_slot_try_reuse(tc, g, (MVMCollectable*)how_obj);
         MVM_spesh_get_facts(tc, g, ins->operands[1])->usages--;
         ins->info = MVM_op_get_op(MVM_OP_sp_getspeshslot);
         ins->operands[1].lit_i16 = spesh_slot;
@@ -214,6 +225,99 @@ static void optimize_isconcrete(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpesh
     }
 }
 
+static void optimize_exception_ops(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb, MVMSpeshIns *ins) {
+    MVMuint16 op = ins->info->opcode;
+
+    if (op == MVM_OP_newexception) {
+        MVMSpeshOperand target   = ins->operands[0];
+        MVMObject      *type     = tc->instance->boot_types.BOOTException;
+        MVMSTable      *st       = STABLE(type);
+        ins->info                = MVM_op_get_op(MVM_OP_sp_fastcreate);
+        ins->operands            = MVM_spesh_alloc(tc, g, 3 * sizeof(MVMSpeshOperand));
+        ins->operands[0]         = target;
+        ins->operands[1].lit_i16 = st->size;
+        ins->operands[2].lit_i16 = MVM_spesh_add_spesh_slot(tc, g, (MVMCollectable *)st);
+    } else {
+        MVMSpeshFacts *target_facts;
+
+        /* XXX This currently still causes problems. */
+        return;
+
+        /*
+        switch (op) {
+        case MVM_OP_bindexmessage:
+        case MVM_OP_bindexpayload: {
+            MVMSpeshOperand target   = ins->operands[0];
+            MVMSpeshOperand value    = ins->operands[1];
+            target_facts             = MVM_spesh_get_facts(tc, g, target);
+
+            if (!(target_facts->flags & MVM_SPESH_FACT_KNOWN_TYPE)
+                || !(REPR(target_facts->type)->ID == MVM_REPR_ID_MVMException))
+                break;
+
+            ins->info                = MVM_op_get_op(op == MVM_OP_bindexmessage ? MVM_OP_sp_bind_s : MVM_OP_sp_bind_o);
+            ins->operands            = MVM_spesh_alloc(tc, g, 3 * sizeof(MVMSpeshOperand));
+            ins->operands[0]         = target;
+            ins->operands[1].lit_i16 = op == MVM_OP_bindexmessage ? offsetof(MVMException, body.message)
+                                                                  : offsetof(MVMException, body.payload);
+            ins->operands[2]         = value;
+            break;
+        }
+        case MVM_OP_bindexcategory: {
+            MVMSpeshOperand target   = ins->operands[0];
+            MVMSpeshOperand category = ins->operands[1];
+            target_facts             = MVM_spesh_get_facts(tc, g, target);
+
+            if (!(target_facts->flags & MVM_SPESH_FACT_KNOWN_TYPE)
+                || !(REPR(target_facts->type)->ID == MVM_REPR_ID_MVMException))
+                break;
+
+            ins->info                = MVM_op_get_op(MVM_OP_sp_bind_i32);
+            ins->operands            = MVM_spesh_alloc(tc, g, 3 * sizeof(MVMSpeshOperand));
+            ins->operands[0]         = target;
+            ins->operands[1].lit_i16 = offsetof(MVMException, body.category);
+            ins->operands[2]         = category;
+            break;
+        }
+        case MVM_OP_getexmessage:
+        case MVM_OP_getexpayload: {
+            MVMSpeshOperand destination = ins->operands[0];
+            MVMSpeshOperand target      = ins->operands[1];
+            target_facts                = MVM_spesh_get_facts(tc, g, target);
+
+            if (!(target_facts->flags & MVM_SPESH_FACT_KNOWN_TYPE)
+                || !(REPR(target_facts->type)->ID == MVM_REPR_ID_MVMException))
+                break;
+
+            ins->info                = MVM_op_get_op(op == MVM_OP_getexmessage ? MVM_OP_sp_get_s : MVM_OP_sp_get_o);
+            ins->operands            = MVM_spesh_alloc(tc, g, 3 * sizeof(MVMSpeshOperand));
+            ins->operands[0]         = destination;
+            ins->operands[1]         = target;
+            ins->operands[2].lit_i16 = op == MVM_OP_getexmessage ? offsetof(MVMException, body.message)
+                                                                 : offsetof(MVMException, body.payload);
+            break;
+        }
+        case MVM_OP_getexcategory: {
+            MVMSpeshOperand destination = ins->operands[0];
+            MVMSpeshOperand target      = ins->operands[1];
+            target_facts                = MVM_spesh_get_facts(tc, g, target);
+
+            if (!(target_facts->flags & MVM_SPESH_FACT_KNOWN_TYPE)
+                || !(REPR(target_facts->type)->ID == MVM_REPR_ID_MVMException))
+                break;
+
+            ins->info                = MVM_op_get_op(MVM_OP_sp_get_i32);
+            ins->operands            = MVM_spesh_alloc(tc, g, 3 * sizeof(MVMSpeshOperand));
+            ins->operands[0]         = destination;
+            ins->operands[1]         = target;
+            ins->operands[2].lit_i16 = offsetof(MVMException, body.category);
+            break;
+        }
+        }
+        */
+    }
+}
+
 /* iffy ops that operate on a known value register can turn into goto
  * or be dropped. */
 static void optimize_iffy(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshIns *ins, MVMSpeshBB *bb) {
@@ -813,7 +917,7 @@ static void optimize_getlex_known(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpe
             MVMSpeshFacts *facts;
 
             /* Place in a spesh slot. */
-            MVMuint16 ss = MVM_spesh_add_spesh_slot(tc, g, log_obj);
+            MVMuint16 ss = MVM_spesh_add_spesh_slot_try_reuse(tc, g, log_obj);
 
             /* Delete logging instruction. */
             MVM_spesh_manipulate_delete_ins(tc, g, bb, ins->next);
@@ -971,7 +1075,7 @@ static void optimize_call(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb
             ss_ins->info        = MVM_op_get_op(MVM_OP_sp_getspeshslot);
             ss_ins->operands    = MVM_spesh_alloc(tc, g, 2 * sizeof(MVMSpeshOperand));
             ss_ins->operands[0] = ins->operands[callee_idx];
-            ss_ins->operands[1].lit_i16 = MVM_spesh_add_spesh_slot(tc, g,
+            ss_ins->operands[1].lit_i16 = MVM_spesh_add_spesh_slot_try_reuse(tc, g,
                 (MVMCollectable *)target);
             /* Basically, we're inserting between arg* and invoke_*.
              * Since invoke_* directly uses the code in the register,
@@ -1088,7 +1192,7 @@ static void optimize_throwcat(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB
         MVMSpeshBB **goto_bbs    = MVM_calloc(g->sf->body.num_handlers, sizeof(MVMSpeshBB *));
         MVMSpeshBB  *search_bb   = g->entry;
         MVMint32     picked      = -1;
-        while (search_bb) {
+        while (search_bb && !search_bb->inlined) {
             MVMSpeshIns *search_ins = search_bb->first_ins;
             while (search_ins) {
                 /* Track handlers. */
@@ -1102,9 +1206,11 @@ static void optimize_throwcat(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB
                         in_handlers[ann->data.frame_handler_index] = 0;
                         break;
                     case MVM_SPESH_ANN_FH_GOTO:
-                        goto_bbs[ann->data.frame_handler_index] = search_bb;
-                        if (picked >= 0 && ann->data.frame_handler_index == picked)
-                            goto search_over;
+                        if (ann->data.frame_handler_index < g->sf->body.num_handlers) {
+                            goto_bbs[ann->data.frame_handler_index] = search_bb;
+                            if (picked >= 0 && ann->data.frame_handler_index == picked)
+                                goto search_over;
+                        }
                         break;
                     }
                     ann = ann->next;
@@ -1314,6 +1420,13 @@ static void optimize_bb(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb)
         case MVM_OP_box_s:
             optimize_repr_op(tc, g, bb, ins, 2);
             break;
+        case MVM_OP_newexception:
+        case MVM_OP_bindexmessage:
+        case MVM_OP_bindexpayload:
+        case MVM_OP_getexmessage:
+        case MVM_OP_getexpayload:
+            optimize_exception_ops(tc, g, bb, ins);
+            break;
         case MVM_OP_hllize:
             optimize_hllize(tc, g, ins);
             break;
