@@ -23,7 +23,7 @@ static void check_strand_sanity(MVMThreadContext *tc, MVMString *s) {
 #endif
 
 /* Checks a string is not null or non-concrete and throws if so. */
-MVM_STATIC_INLINE MVM_string_check_arg(MVMThreadContext *tc, MVMString *s, const char *operation) {
+MVM_STATIC_INLINE void MVM_string_check_arg(MVMThreadContext *tc, MVMString *s, const char *operation) {
     if (!s || !IS_CONCRETE(s))
         MVM_exception_throw_adhoc(tc, "%s requires a concrete string, but got %s",
             operation, s ? "a type object" : "null");
